@@ -11,7 +11,7 @@ cd tsrc
 PATH=$PATH:/opt/mingw/bin
 TCLDIR=/home/drh/tcltk/846/win/846win
 TCLSTUBLIB=$TCLDIR/libtcl84stub.a
-OPTS='-DUSE_TCL_STUBS=1 -DNDEBUG=1 -DTHREADSAFE=1'
+OPTS='-DUSE_TCL_STUBS=1 -DNDEBUG=1 -DTHREADSAFE=1 -DBUILD_sqlite=1'
 CC="i386-mingw32msvc-gcc -O2 $OPTS -I. -I$TCLDIR"
 rm shell.c
 for i in *.c; do
@@ -31,7 +31,7 @@ i386-mingw32msvc-dllwrap \
      --as i386-mingw32msvc-as \
      --target i386-mingw32 \
      -dllname tclsqlite3.dll -lmsvcrt *.o $TCLSTUBLIB
-i386-mingw32msvc-strip tclsqlite3.dll
+#i386-mingw32msvc-strip tclsqlite3.dll
 rm tclsqlite.o
 i386-mingw32msvc-dllwrap \
      --def sqlite3.def -v --export-all \
@@ -40,5 +40,5 @@ i386-mingw32msvc-dllwrap \
      --as i386-mingw32msvc-as \
      --target i386-mingw32 \
      -dllname sqlite3.dll -lmsvcrt *.o
-i386-mingw32msvc-strip sqlite3.dll
+#i386-mingw32msvc-strip sqlite3.dll
 cd ..
