@@ -11,7 +11,7 @@
 # This file implements some common TCL routines used for regression
 # testing the SQLite library
 #
-# $Id: tester.tcl,v 1.50 2005/09/17 15:20:28 drh Exp $
+# $Id: tester.tcl,v 1.64 2006/01/24 13:09:33 danielk1977 Exp $
 
 # Make sure tclsqlite3 was compiled correctly.  Abort now with an
 # error message if not.
@@ -61,7 +61,8 @@ if {[sqlite3 -has-codec] && [info command sqlite_orig]==""} {
 catch {db close}
 file delete -force test.db
 file delete -force test.db-journal
-set ::DB [sqlite3 db ./test.db]
+sqlite3 db ./test.db
+set ::DB [sqlite3_connection_pointer db]
 if {[info exists ::SETUP_SQL]} {
   db eval $::SETUP_SQL
 }
@@ -83,6 +84,7 @@ set maxErr 1000
 #
 proc do_test {name cmd expected} {
   global argv nErr nTest skip_test maxErr
+  set ::sqlite_malloc_id $name
   if {$skip_test} {
     set skip_test 0
     return
@@ -139,7 +141,24 @@ proc finish_test {} {
 proc finalize_testing {} {
   global nTest nErr nProb sqlite_open_file_count
   if {$nErr==0} memleak_check
+
   catch {db close}
+  catch {db2 close}
+  catch {db3 close}
+
+  catch {
+    pp_check_for_leaks
+  }
+  sqlite3 db {}
+  # sqlite3_clear_tsd_memdebug
+  db close
+  if {$::sqlite3_tsd_count} {
+     puts "Thread-specific data leak: $::sqlite3_tsd_count instances"
+     incr nErr
+  } else {
+     puts "Thread-specific data deallocated properly"
+  }
+  incr nTest
   puts "$nErr errors out of $nTest tests"
   puts "Failures on these tests: $::failList"
   if {$nProb>0} {
@@ -159,7 +178,7 @@ proc finalize_testing {} {
 #
 proc execsql {sql {db db}} {
   # puts "SQL = $sql"
-  return [$db eval $sql]
+  uplevel [list $db eval $sql]
 }
 
 # Execute SQL and catch exceptions.
@@ -281,7 +300,7 @@ proc crashsql {crashdelay crashfile sql} {
   close $f
 
   set r [catch {
-    exec [file join . crashtest] crash.tcl >@stdout
+    exec [info nameofexec] crash.tcl >@stdout
   } msg]
   lappend r $msg
 }
@@ -326,7 +345,7 @@ proc do_ioerr_test {testname args} {
       catch {file delete -force test.db-journal}
       catch {file delete -force test2.db}
       catch {file delete -force test2.db-journal}
-      set ::DB [sqlite3 db test.db]
+      set ::DB [sqlite3 db test.db; sqlite3_connection_pointer db]
       if {[info exists ::ioerropts(-tclprep)]} {
         eval $::ioerropts(-tclprep)
       }
@@ -362,9 +381,11 @@ proc do_ioerr_test {testname args} {
     do_test $testname.$n.3 {
       set r [catch $::ioerrorbody msg]
       set ::go [expr {$::sqlite_io_error_pending<=0}]
-      set s [expr $::sqlite_io_error_pending>0]
+      set s [expr $::sqlite_io_error_hit==0]
+      set ::sqlite_io_error_hit 0
       # puts "$::sqlite_io_error_pending $r $msg"
-      expr { ($s && !$r) || (!$s && $r) }
+      # puts "r=$r s=$s go=$::go msg=\"$msg\""
+      expr { ($s && !$r && !$::go) || (!$s && $r && $::go) }
       # expr {$::sqlite_io_error_pending>0 || $r!=0}
     } {1}
 
@@ -373,11 +394,12 @@ proc do_ioerr_test {testname args} {
     if {$::go && $::ioerropts(-cksum)} {
       do_test $testname.$n.4 {
         catch {db close}
-        set ::DB [sqlite3 db test.db]
+        set ::DB [sqlite3 db test.db; sqlite3_connection_pointer db]
         cksum
       } $checksum
     }
 
+    set ::sqlite_io_error_pending 0
     if {[info exists ::ioerropts(-cleanup)]} {
       catch $::ioerropts(-cleanup)
     }
@@ -423,6 +445,63 @@ proc copy_file {from to} {
   }
 }
 
+# This command checks for outstanding calls to sqliteMalloc() from within
+# the current thread. A list is returned with one entry for each outstanding
+# malloc. Each list entry is itself a list of 5 items, as follows:
+#
+#     { <number-bytes> <file-name> <line-number> <test-case> <stack-dump> }
+#
+proc check_for_leaks {} {
+  set ret [list]
+  set cnt 0
+  foreach alloc [sqlite_malloc_outstanding] {
+    foreach {nBytes file iLine userstring backtrace} $alloc {}
+    set stack [list]
+    set skip 0
+
+    # The first command in this block will probably fail on windows. This
+    # means there will be no stack dump available.
+    if {$cnt < 25 && $backtrace!=""} {
+      catch {
+        set stuff [eval "exec addr2line -e ./testfixture -f $backtrace"]
+        foreach {func line} $stuff {
+          if {$func != "??" || $line != "??:0"} {
+            regexp {.*/(.*)} $line dummy line
+            lappend stack "${func}() $line"
+          } else {
+            if {[lindex $stack end] != "..."} {
+              lappend stack "..."
+            }
+          }
+        }
+      }
+      incr cnt
+    }
+
+    if {!$skip} {
+      lappend ret [list $nBytes $file $iLine $userstring $stack]
+    }
+  }
+  return $ret
+}
+
+# Pretty print a report based on the return value of [check_for_leaks] to
+# stdout.
+proc pp_check_for_leaks {} {
+  set l [check_for_leaks]
+  set n 0
+  foreach leak $l {
+    foreach {nBytes file iLine userstring stack} $leak {}
+    puts "$nBytes bytes leaked at $file:$iLine ($userstring)"
+    foreach frame $stack {
+      puts "        $frame"
+    }
+    incr n $nBytes
+  }
+  puts "Memory leaked: $n bytes in [llength $l] allocations"
+  puts ""
+}
+
 # If the library is compiled with the SQLITE_DEFAULT_AUTOVACUUM macro set
 # to non-zero, then set the global variable $AUTOVACUUM to 1.
 set AUTOVACUUM $sqlite_options(default_autovacuum)
