@@ -1,4 +1,4 @@
-set rcsid {$Id: capi3ref.tcl,v 1.20 2005/03/14 02:01:50 drh Exp $}
+set rcsid {$Id: capi3ref.tcl,v 1.38 2006/04/05 01:08:35 drh Exp $}
 source common.tcl
 header {C/C++ Interface For SQLite Version 3}
 puts {
@@ -72,9 +72,12 @@ api {} {
 api {} {
   int sqlite3_aggregate_count(sqlite3_context*);
 } {
-  The next routine returns the number of calls to xStep for a particular
-  aggregate function instance.  The current call to xStep counts so this
-  routine always returns at least 1.
+  This function is deprecated.   It continues to exist so as not to
+  break any legacy code that might happen to use it.  But it should not
+  be used in any new code.
+
+  In order to encourage people to not use this function, we are not going
+  to tell you what it does.
 }
 
 api {} {
@@ -104,6 +107,16 @@ api {} {
  named parameters can be looked up using the
  sqlite3_bind_parameter_name() API if desired.
 
+ The third argument is the value to bind to the parameter.
+
+ In those
+ routines that have a fourth argument, its value is the number of bytes
+ in the parameter.  To be clear: the value is the number of bytes in the
+ string, not the number of characters.  The number
+ of bytes does not include the zero-terminator at the end of strings.
+ If the fourth parameter is negative, the length of the string is
+ number of bytes up to the first zero terminator.
+
  The fifth argument to sqlite3_bind_blob(), sqlite3_bind_text(), and
  sqlite3_bind_text16() is a destructor used to dispose of the BLOB or
  text after SQLite has finished with it.  If the fifth argument is the
@@ -116,6 +129,12 @@ api {} {
  sqlite3_prepare() or sqlite3_reset() and before sqlite3_step().
  Bindings are not cleared by the sqlite3_reset() routine.
  Unbound parameters are interpreted as NULL.
+
+ These routines return SQLITE_OK on success or an error code if
+ anything goes wrong.  SQLITE_RANGE is returned if the parameter
+ index is out of range.  SQLITE_NOMEM is returned if malloc fails.
+ SQLITE_MISUSE is returned if these routines are called on a virtual
+ machine that is the wrong state or which has already been finalized.
 }
 
 api {} {
@@ -157,7 +176,7 @@ api {} {
  If the busy callback is NULL, then SQLITE_BUSY is returned immediately
  upon encountering the lock.
  If the busy callback is not NULL, then the
- callback might invoked with two arguments.  The
+ callback will be invoked with two arguments.  The
  second argument is the number of prior calls to the busy callback
  for the same lock.  If the
  busy callback returns 0, then no additional attempts are made to
@@ -312,8 +331,8 @@ int sqlite3_column_type(sqlite3_stmt*, int iCol);
 <tr><th>Internal Type</th><th>Requested Type</th><th>Conversion</th></tr>
 <tr><td> NULL    </td><td> INTEGER</td><td>Result is 0</td></tr>
 <tr><td> NULL </td><td>    FLOAT </td><td> Result is 0.0</td></tr>
-<tr><td> NULL </td><td>    TEXT </td><td>  Result is an empty string</td></tr>
-<tr><td> NULL </td><td>    BLOB </td><td>  Result is a zero-length BLOB</td></tr>
+<tr><td> NULL </td><td>    TEXT </td><td>  Result is NULL pointer</td></tr>
+<tr><td> NULL </td><td>    BLOB </td><td>  Result is NULL pointer</td></tr>
 <tr><td> INTEGER </td><td> FLOAT </td><td> Convert from integer to float</td></tr>
 <tr><td> INTEGER </td><td> TEXT </td><td>  ASCII rendering of the integer</td></tr>
 <tr><td> INTEGER </td><td> BLOB </td><td>  Same as for INTEGER->TEXT</td></tr>
@@ -347,11 +366,10 @@ const void *sqlite3_column_decltype16(sqlite3_stmt*,int);
  The first argument is a prepared SQL statement. If this statement
  is a SELECT statement, the Nth column of the returned result set 
  of the SELECT is a table column then the declared type of the table
- column is returned. If the Nth column of the result set is not at table
+ column is returned. If the Nth column of the result set is not a table
  column, then a NULL pointer is returned. The returned string is 
  UTF-8 encoded for sqlite3_column_decltype() and UTF-16 encoded
- for sqlite3_column_decltype16().
- For example, in the database schema:
+ for sqlite3_column_decltype16(). For example, in the database schema:
 
  <blockquote><pre>
  CREATE TABLE t1(c1 INTEGER);
@@ -360,12 +378,140 @@ const void *sqlite3_column_decltype16(sqlite3_stmt*,int);
  And the following statement compiled:
 
  <blockquote><pre>
- SELECT c1 + 1, 0 FROM t1;
+ SELECT c1 + 1, c1 FROM t1;
  </pre></blockquote>
 
  Then this routine would return the string "INTEGER" for the second
  result column (i==1), and a NULL pointer for the first result column
  (i==0).
+
+ If the following statements were compiled then this routine would
+ return "INTEGER" for the first (only) result column.
+
+ <blockquote><pre>
+ SELECT (SELECT c1) FROM t1;
+ SELECT (SELECT c1 FROM t1);
+ SELECT c1 FROM (SELECT c1 FROM t1);
+ SELECT * FROM (SELECT c1 FROM t1);
+ SELECT * FROM (SELECT * FROM t1);
+ </pre></blockquote>
+}
+
+api {} {
+  int sqlite3_table_column_metadata(
+    sqlite3 *db,                /* Connection handle */
+    const char *zDbName,        /* Database name or NULL */
+    const char *zTableName,     /* Table name */
+    const char *zColumnName,    /* Column name */
+    char const **pzDataType,    /* OUTPUT: Declared data type */
+    char const **pzCollSeq,     /* OUTPUT: Collation sequence name */
+    int *pNotNull,              /* OUTPUT: True if NOT NULL constraint exists */
+    int *pPrimaryKey,           /* OUTPUT: True if column part of PK */
+    int *pAutoinc               /* OUTPUT: True if colums is auto-increment */
+  );
+} {
+ This routine is used to obtain meta information about a specific column of a
+ specific database table accessible using the connection handle passed as the
+ first function argument.
+
+ The column is identified by the second, third and fourth parameters to 
+ this function. The second parameter is either the name of the database
+ (i.e. "main", "temp" or an attached database) containing the specified
+ table or NULL. If it is NULL, then all attached databases are searched
+ for the table using the same algorithm as the database engine uses to 
+ resolve unqualified table references.
+
+ The third and fourth parameters to this function are the table and column 
+ name of the desired column, respectively. Neither of these parameters 
+ may be NULL.
+
+ Meta information is returned by writing to the memory locations passed as
+ the 5th and subsequent parameters to this function. Any of these 
+ arguments may be NULL, in which case the corresponding element of meta 
+ information is ommitted.
+
+<pre>
+ Parameter     Output Type      Description
+ -----------------------------------
+   5th         const char*      Declared data type 
+   6th         const char*      Name of the columns default collation sequence 
+   7th         int              True if the column has a NOT NULL constraint
+   8th         int              True if the column is part of the PRIMARY KEY
+   9th         int              True if the column is AUTOINCREMENT
+</pre>
+
+ The memory pointed to by the character pointers returned for the 
+ declaration type and collation sequence is valid only until the next 
+ call to any sqlite API function.
+
+ This function may load one or more schemas from database files. If an
+ error occurs during this process, or if the requested table or column
+ cannot be found, an SQLITE error code is returned and an error message
+ left in the database handle (to be retrieved using sqlite3_errmsg()).
+ Specifying an SQL view instead of a table as the third argument is also
+ considered an error.
+
+ If the specified column is "rowid", "oid" or "_rowid_" and an 
+ INTEGER PRIMARY KEY column has been explicitly declared, then the output 
+ parameters are set for the explicitly declared column. If there is no
+ explicitly declared IPK column, then the data-type is "INTEGER", the
+ collation sequence "BINARY" and the primary-key flag is set. Both
+ the not-null and auto-increment flags are clear.
+
+ This API is only available if the library was compiled with the
+ SQLITE_ENABLE_COLUMN_METADATA preprocessor symbol defined.
+}
+
+api {} {
+const char *sqlite3_column_database_name(sqlite3_stmt *pStmt, int N);
+const void *sqlite3_column_database_name16(sqlite3_stmt *pStmt, int N);
+} {
+If the Nth column returned by statement pStmt is a column reference,
+these functions may be used to access the name of the database (either 
+"main", "temp" or the name of an attached database) that contains
+the column. If the Nth column is not a column reference, NULL is
+returned.
+
+See the description of function sqlite3_column_decltype() for a
+description of exactly which expressions are considered column references.
+
+Function sqlite3_column_database_name() returns a pointer to a UTF-8
+encoded string. sqlite3_column_database_name16() returns a pointer
+to a UTF-16 encoded string. 
+}
+
+api {} {
+const char *sqlite3_column_origin_name(sqlite3_stmt *pStmt, int N);
+const void *sqlite3_column_origin_name16(sqlite3_stmt *pStmt, int N);
+} {
+If the Nth column returned by statement pStmt is a column reference,
+these functions may be used to access the schema name of the referenced 
+column in the database schema. If the Nth column is not a column 
+reference, NULL is returned.
+
+See the description of function sqlite3_column_decltype() for a
+description of exactly which expressions are considered column references.
+
+Function sqlite3_column_origin_name() returns a pointer to a UTF-8
+encoded string. sqlite3_column_origin_name16() returns a pointer
+to a UTF-16 encoded string. 
+}
+
+api {} {
+const char *sqlite3_column_table_name(sqlite3_stmt *pStmt, int N);
+const void *sqlite3_column_table_name16(sqlite3_stmt *pStmt, int N);
+} {
+If the Nth column returned by statement pStmt is a column reference, 
+these functions may be used to access the name of the table that 
+contains the column.  If the Nth column is not a column reference, 
+NULL is returned.
+
+See the description of function sqlite3_column_decltype() for a
+description of exactly which expressions are considered column references.
+
+Function sqlite3_column_table_name() returns a pointer to a UTF-8
+encoded string. sqlite3_column_table_name16() returns a pointer
+to a UTF-16 encoded string. 
 }
 
 api {} {
@@ -836,6 +982,17 @@ int sqlite3_open16(
  Whether or not an error occurs when it is opened, resources associated
  with the sqlite3* handle should be released by passing it to
  sqlite3_close() when it is no longer required.
+
+ The returned sqlite3* can only be used in the same thread in which it
+ was created.  It is an error to call sqlite3_open() in one thread then
+ pass the resulting database handle off to another thread to use.  This
+ restriction is due to goofy design decisions (bugs?) in the way some
+ threading implementations interact with file locks.
+
+ Note to windows users:  The encoding used for the filename argument
+ of sqlite3_open() must be UTF-8, not whatever codepage is currently
+ defined.  Filenames containing international characters must be converted
+ to UTF-8 prior to passing them into sqlite3_open().
 }
 
 api {} {
@@ -976,6 +1133,9 @@ int sqlite3_set_authorizer(
 #define SQLITE_UPDATE               23   /* Table Name      Column Name     */
 #define SQLITE_ATTACH               24   /* Filename        NULL            */
 #define SQLITE_DETACH               25   /* Database Name   NULL            */
+#define SQLITE_ALTER_TABLE          26   /* Database Name   Table Name      */
+#define SQLITE_REINDEX              27   /* Index Name      NULL            */
+#define SQLITE_ANALYZE              28   /* Table Name      NULL            */
 
 #define SQLITE_DENY   1   /* Abort the SQL statement with an error */
 #define SQLITE_IGNORE 2   /* Don't allow access, but don't generate an error */
@@ -1038,8 +1198,8 @@ int sqlite3_step(sqlite3_stmt*);
  SQLITE_MISUSE means that the this routine was called inappropriately.
  Perhaps it was called on a virtual machine that had already been
  finalized or on one that had previously returned SQLITE_ERROR or
- SQLITE_DONE.  Or it could be the case the the same database connection
- is being used simultaneously by two or more threads.
+ SQLITE_DONE.  Or it could be the case that a database connection
+ is being used by a different thread than the one it was created it.
 }
 
 api {} {
@@ -1086,6 +1246,225 @@ int sqlite3_value_type(sqlite3_value*);
  information.
 }
 
+api {} {
+  int sqlite3_sleep(int);
+} {
+ Sleep for a little while. The second parameter is the number of
+ miliseconds to sleep for. 
+
+ If the operating system does not support sleep requests with 
+ milisecond time resolution, then the time will be rounded up to 
+ the nearest second. The number of miliseconds of sleep actually 
+ requested from the operating system is returned.
+}
+
+api {} {
+  int sqlite3_expired(sqlite3_stmt*);
+} {
+ Return TRUE (non-zero) if the statement supplied as an argument needs
+ to be recompiled.  A statement needs to be recompiled whenever the
+ execution environment changes in a way that would alter the program
+ that sqlite3_prepare() generates.  For example, if new functions or
+ collating sequences are registered or if an authorizer function is
+ added or changed.
+}
+
+api {} {
+  int sqlite3_transfer_bindings(sqlite3_stmt*, sqlite3_stmt*);
+} {
+ Move all bindings from the first prepared statement over to the second.
+ This routine is useful, for example, if the first prepared statement
+ fails with an SQLITE_SCHEMA error.  The same SQL can be prepared into
+ the second prepared statement then all of the bindings transfered over
+ to the second statement before the first statement is finalized.
+}
+
+api {} {
+  int sqlite3_global_recover();
+} {
+ This function used to be involved in recovering from out-of-memory
+ errors.  But as of SQLite version 3.3.0, out-of-memory recovery is
+ automatic and this routine now does nothing.  THe interface is retained
+ to avoid link errors with legacy code.
+}
+
+api {} {
+  int sqlite3_get_autocommit(sqlite3*);
+} {
+ Test to see whether or not the database connection is in autocommit
+ mode.  Return TRUE if it is and FALSE if not.  Autocommit mode is on
+ by default.  Autocommit is disabled by a BEGIN statement and reenabled
+ by the next COMMIT or ROLLBACK.
+}
+
+api {} {
+  int sqlite3_clear_bindings(sqlite3_stmt*);
+} {
+ Set all the parameters in the compiled SQL statement back to NULL.
+}
+
+api {} {
+  sqlite3 *sqlite3_db_handle(sqlite3_stmt*);
+} {
+ Return the sqlite3* database handle to which the prepared statement given
+ in the argument belongs.  This is the same database handle that was
+ the first argument to the sqlite3_prepare() that was used to create
+ the statement in the first place.
+}
+
+api {} {
+  void *sqlite3_update_hook(
+    sqlite3*, 
+    void(*)(void *,int ,char const *,char const *,sqlite_int64),
+    void*
+  );
+} {
+ Register a callback function with the database connection identified by the 
+ first argument to be invoked whenever a row is updated, inserted or deleted.
+ Any callback set by a previous call to this function for the same 
+ database connection is overridden.
+
+ The second argument is a pointer to the function to invoke when a 
+ row is updated, inserted or deleted. The first argument to the callback is
+ a copy of the third argument to sqlite3_update_hook. The second callback 
+ argument is one of SQLITE_INSERT, SQLITE_DELETE or SQLITE_UPDATE, depending
+ on the operation that caused the callback to be invoked. The third and 
+ fourth arguments to the callback contain pointers to the database and 
+ table name containing the affected row. The final callback parameter is 
+ the rowid of the row. In the case of an update, this is the rowid after 
+ the update takes place.
+
+ The update hook is not invoked when internal system tables are
+ modified (i.e. sqlite_master and sqlite_sequence).
+
+ If another function was previously registered, its pArg value is returned.
+ Otherwise NULL is returned.
+
+ See also: sqlite3_commit_hook(), sqlite3_rollback_hook()
+}
+
+api {} {
+  void *sqlite3_rollback_hook(sqlite3*, void(*)(void *), void*);
+} {
+ Register a callback to be invoked whenever a transaction is rolled
+ back. 
+
+ The new callback function overrides any existing rollback-hook
+ callback. If there was an existing callback, then it's pArg value 
+ (the third argument to sqlite3_rollback_hook() when it was registered) 
+ is returned. Otherwise, NULL is returned.
+
+ For the purposes of this API, a transaction is said to have been 
+ rolled back if an explicit "ROLLBACK" statement is executed, or
+ an error or constraint causes an implicit rollback to occur. The 
+ callback is not invoked if a transaction is automatically rolled
+ back because the database connection is closed.
+}
+
+api {} {
+  int sqlite3_enable_shared_cache(int);
+} {
+  This routine enables or disables the sharing of the database cache
+  and schema data structures between connections to the same database.
+  Sharing is enabled if the argument is true and disabled if the argument
+  is false.
+
+  Cache sharing is enabled and disabled on a thread-by-thread basis.
+  Each call to this routine enables or disables cache sharing only for
+  connections created in the same thread in which this routine is called.
+  There is no mechanism for sharing cache between database connections
+  running in different threads.
+
+  Sharing must be disabled prior to shutting down a thread or else
+  the thread will leak memory.  Call this routine with an argument of
+  0 to turn off sharing.  Or use the sqlite3_thread_cleanup() API.
+
+  This routine must not be called when any database connections
+  are active in the current thread.  Enabling or disabling shared
+  cache while there are active database connections will result
+  in memory corruption.
+
+  When the shared cache is enabled, the
+  following routines must always be called from the same thread:
+  sqlite3_open(), sqlite3_prepare(), sqlite3_step(), sqlite3_reset(),
+  sqlite3_finalize(), and sqlite3_close().
+  This is due to the fact that the shared cache makes use of
+  thread-specific storage so that it will be available for sharing
+  with other connections.
+
+  This routine returns SQLITE_OK if shared cache was
+  enabled or disabled successfully.  An error code is returned
+  otherwise.
+
+  Shared cache is disabled by default for backward compatibility.
+}
+
+api {} {
+  void sqlite3_thread_cleanup(void);
+} {
+  This routine makes sure that all thread local storage used by SQLite
+  in the current thread has been deallocated.  A thread can call this
+  routine prior to terminating in order to make sure there are no memory
+  leaks.
+
+  This routine is not strictly necessary.  If cache sharing has been
+  disabled using sqlite3_enable_shared_cache() and if all database
+  connections have been closed and if SQLITE_ENABLE_MEMORY_MANAGMENT is
+  on and all memory has been freed, then the thread local storage will
+  already have been automatically deallocated.  This routine is provided
+  as a convenience to the program who just wants to make sure that there
+  are no leaks.
+}
+
+api {} {
+  int sqlite3_release_memory(int N);
+} {
+  This routine attempts to free at least N bytes of memory from the caches
+  of database connecions that were created in the same thread from which this
+  routine is called.  The value returned is the number of bytes actually
+  freed.  
+
+  This routine is only available if memory management has been enabled
+  by compiling with the SQLITE_ENABLE_MEMORY_MANAGMENT macro.
+}
+
+api {} {
+  void sqlite3_soft_heap_limit(int N);
+} {
+  This routine sets the soft heap limit for the current thread to N.
+  If the total heap usage by SQLite in the current thread exceeds N,
+  then sqlite3_release_memory() is called to try to reduce the memory usage
+  below the soft limit.
+
+  Prior to shutting down a thread sqlite3_soft_heap_limit() must be set to 
+  zero (the default) or else the thread will leak memory. Alternatively, use
+  the sqlite3_thread_cleanup() API.
+
+  A negative or zero value for N means that there is no soft heap limit and
+  sqlite3_release_memory() will only be called when memory is exhaused.
+  The default value for the soft heap limit is zero.
+
+  SQLite makes a best effort to honor the soft heap limit.  But if it
+  is unable to reduce memory usage below the soft limit, execution will
+  continue without error or notification.  This is why the limit is 
+  called a "soft" limit.  It is advisory only.
+
+  This routine is only available if memory management has been enabled
+  by compiling with the SQLITE_ENABLE_MEMORY_MANAGMENT macro.
+}
+
+api {} {
+  void sqlite3_thread_cleanup(void);
+} {
+  This routine ensures that a thread that has used SQLite in the past
+  has released any thread-local storage it might have allocated.  
+  When the rest of the API is used properly, the cleanup of 
+  thread-local storage should be completely automatic.  You should
+  never really need to invoke this API.  But it is provided to you
+  as a precaution and as a potential work-around for future
+  thread-releated memory-leaks.
+}
+
 set n 0
 set i 0
 foreach item $apilist {
@@ -1103,6 +1482,11 @@ foreach name [lsort [array names name_to_idx]] {
   set sname($i) $name
   incr i
 }
+#parray n_to_name
+#parray n_to_idx
+#parray name_to_idx
+#parray sname
+incr n -1
 puts {<table width="100%" cellpadding="5"><tr>}
 set nrow [expr {($n+2)/3}]
 set i 0
