@@ -1,7 +1,7 @@
 #
 # Run this script to generated a faq.html output file
 #
-set rcsid {$Id: faq.tcl,v 1.31 2005/09/17 02:34:05 drh Exp $}
+set rcsid {$Id: faq.tcl,v 1.36 2006/04/05 01:02:08 drh Exp $}
 source common.tcl
 header {SQLite Frequently Asked Questions</title>}
 
@@ -136,7 +136,7 @@ faq {
   <p>Multiple processes can have the same database open at the same
   time.  Multiple processes can be doing a SELECT
   at the same time.  But only one process can be making changes to
-  the database at an given moement in time, however.</p>
+  the database at any moment in time, however.</p>
 
   <p>SQLite uses reader/writer locks to control access to the database.
   (Under Win95/98/ME which lacks support for reader/writer locks, a
@@ -197,15 +197,24 @@ faq {
   to use the same <b>sqlite3</b> structure pointer in two
   or more threads.</p>
 
-  <p>An <b>sqlite3</b> structure can only be used in the same thread
+  <p>Prior to version 3.3.1,
+  an <b>sqlite3</b> structure could only be used in the same thread
   that called <a href="capi3ref#sqlite3_open">sqlite3_open</a> to create it.
-  You cannot open a
+  You could not open a
   database in one thread then pass the handle off to another thread for
-  it to use.  This is due to limitations (bugs?) in many common threading
-  implementations such as on RedHat9.  There may be ways to work around
-  these limitations, but they are complex and exceedingly difficult to
-  test for correctness.  For that reason, SQLite currently takes the safe
-  approach and disallows the sharing of handles among threads.</p>
+  it to use.  This was due to limitations (bugs?) in many common threading
+  implementations such as on RedHat9.  Specifically, an fcntl() lock
+  created by one thread cannot be removed or modified by a different
+  thread on the troublesome systems.  And since SQLite uses fcntl()
+  locks heavily for concurrency control, serious problems arose if you 
+  start moving database connections across threads.</p>
+
+  <p>The restriction on moving database connections across threads
+  was relaxed somewhat in version 3.3.1.  With that and subsequent
+  versions, it is safe to move a connection handle across threads
+  as long as the connection is not holding any fcntl() locks.  You
+  can safely assume that no locks are being held if no
+  transaction is pending and all statements have been finalized.</p>
 
   <p>Under UNIX, you should not carry an open SQLite database across
   a fork() system call into the child process.  Problems will result
@@ -318,8 +327,8 @@ faq {
 faq {
   Does SQLite support a BLOB type?
 } {
-  <p>SQLite versions 3.0 and leter let you puts BLOB data into any column, even
-  columns that are declared to hold some other type.</p>
+  <p>SQLite versions 3.0 and later allow you to store BLOB data in any 
+  column, even columns that are declared to hold some other type.</p>
 }
 
 faq {
